
#ifndef HAPTICORE_LIBRARY
#define HAPTICORE_LIBRARY

#include <Arduino.h>

#include "HapticoreConversionFactors.h"
#include "HapticoreTypes.h"

namespace HapticoreLibrary
{
	union RegisterData
	{
		byte BytesType[2];
		float FloatType;
		short int SignedType;
		unsigned short int UnsignedType;
		char* StringType;
	};

	class HapticoreDevice
	{
		private:
			typedef void (*DataReceivedHandler_t)(const Register reg, const RegisterData data);
			typedef void (*OutputMsgHandler_t)(const String message);
			
			HardwareSerial* SerialPort;
			
			DataReceivedHandler_t DataReceivedHandler;
			OutputMsgHandler_t PrintMsgHandler;

			byte parseBuffer[6];
			int parseBufferIdx = -1;

			void NotifyDataReceivedSafe(const Register reg, const RegisterData data);
			void PrintMsgSafe(const String message);

			unsigned short int ConvertDataToUshort(const byte* const packet);
			RegisterData RequestBytesType(const byte* const packet);
			RegisterData RequestUshortFloatType(const byte* const packet, unsigned short int conversionFactor);
			RegisterData RequestShortFloatType(const byte* const packet, unsigned short int conversionFactor);
			RegisterData RequestSignedType(const byte* const packet);
			RegisterData RequestUnsignedType(const byte* const packet);

			byte* SendBytesType(const RegisterData data);
			byte* SendUnsignedType(const RegisterData data);
			byte* SendUshortFloatType(const RegisterData data, const unsigned short int conversionFactor);

			void ParseReceivedByte(const byte data);
			void HandlePacket (const byte* const packet);

			void SerialWrite(const Register reg, const byte dataHighByte, const byte dataLowByte);
			void ClearState();
		
		public:
			void Connect(HardwareSerial* port, const long baudRate);
			void Disconnect();
			void PollData();
			
			void Request(const Register reg);
			void Send(const Register reg, const RegisterData data);

			void AttachDataReceivedHandler(const DataReceivedHandler_t handler);
			void AttachDebugLogHandler(const OutputMsgHandler_t handler);
			void DetachHandler();
	};
}

#endif
